function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import React, { useState } from 'react';
import { useValidation } from './useValidation';
var defaultProps = {
  rules: [],
  Wrapper: React.Fragment,
  InputComponent: 'input',
  type: 'text',
  bindEventListener: ['onChange'],
  bindValidityToEvent: [],
  children: function children() {
    return null;
  },
  defaultValue: undefined,
  validateOnMount: false,
  onValidityChanged: function onValidityChanged() {},
  onValiditySuccess: function onValiditySuccess() {},
  onValidityFailure: function onValidityFailure() {}
};
/**
 * Higher order component that attaches the `useValidation` hook to the provided input component.
 * Default input component is set to `<input type="text" />` and binds the validation call to `onChange` event listener.
 *
 * Important props:
 * - **rules**: ValidationRule[] - an array of validation rules
 * - **InputComponent**: React.ElementType<any> | React.FC<any> - specifies what input component to use. Default: `<input />`
 * - **Wrapper**: React.ElementType<any> - specifies wrapper component around the input component. Default: `React.Fragment`
 * - **inputValueParser**: function - function used to specify how to get the input value from the input component.
 *  Pulls from `event.currentTarget.value` if undefined. Default: `undefined`
 * - **bindEventListener**: string[] - specifies which event listener to bind the validation call to and also return the validation state as 2nd argument to event handler.
 * Default: `["onChange"]`
 * - **bindValidityToEvent**: string[] - specifies which event listener to attach the validation state to (2nd argument) ***WITHOUT*** calling the validation function.
 * This has lower priority than `bindEventListener` if there is an overlap of events. Default: `[]`
 * - **validateOnMount**: boolean - validates the input on component mount when true. Default: `false`
 * - **onValidityChange**: (validity: ValidationState, inputValue: any, name?: string) => void - an event listener that triggers when validation state (`validity.passed`) changes
 * - **onValiditySuccess**: (validity: ValidationState, inputValue: any, name?: string) => void - an event listener that triggers when validation state (`validity.passed`) evaluates to true
 * - **onValidityFailure**: (validity: ValidationState, inputValue: any, name?: string) => void - an event listener that triggers when validation state (`validity.passed`) evaluates to false
 * - **children**: (validity: ValidationState, validate: ValidateFn) => React.ReactNode - Specifies additional elements to be rendered after the input component. Must be a function. Default: `undefined`.
 * The `validate` fn is to provide a manual way of calling the validation on that input component.
 *
 * The rest of the props are passed as-is into the input component
 * Note: `props.type` is defaulted to `"text"`
 * Note: `props.defaultValue` is used as the 1st input value to validate against
 *
 * @param props ValidationInputProps
 * @returns
 */

var ValidateInput = function ValidateInput(_ref) {
  var rules = _ref.rules,
      defaultValue = _ref.defaultValue,
      Wrapper = _ref.Wrapper,
      InputComponent = _ref.InputComponent,
      inputValueParser = _ref.inputValueParser,
      bindEventListener = _ref.bindEventListener,
      bindValidityToEvent = _ref.bindValidityToEvent,
      validateOnMount = _ref.validateOnMount,
      onValidityChanged = _ref.onValidityChanged,
      onValiditySuccess = _ref.onValiditySuccess,
      onValidityFailure = _ref.onValidityFailure,
      children = _ref.children,
      props = _objectWithoutProperties(_ref, ["rules", "defaultValue", "Wrapper", "InputComponent", "inputValueParser", "bindEventListener", "bindValidityToEvent", "validateOnMount", "onValidityChanged", "onValiditySuccess", "onValidityFailure", "children"]);

  var _useState = useState(defaultValue),
      _useState2 = _slicedToArray(_useState, 2),
      inputValue = _useState2[0],
      setInputValue = _useState2[1];

  var _useValidation = useValidation(inputValue, rules, {
    inputName: props.name,
    validateOnMount: validateOnMount,
    onValidityChanged: onValidityChanged,
    onValiditySuccess: onValiditySuccess,
    onValidityFailure: onValidityFailure
  }),
      validity = _useValidation.validity,
      validate = _useValidation.validate;
  /**
   * Generates a map of event handlers that will call the validate fn and the original event handler and attaches validation state
   * as the 2nd argument
   */


  var generateValidationEvent = function generateValidationEvent() {
    var validateOnEvent = {};
    bindEventListener.forEach(function (event) {
      validateOnEvent[event] = attachedEventListener(event);
    });
    return validateOnEvent;
  };
  /**
   * Generates a map of event handlers that will attach the validation state (validity) to original event handler as 2nd argument
   */


  var generateAppendValidityEvent = function generateAppendValidityEvent() {
    var validityForEvent = {};
    bindValidityToEvent.forEach(function (event) {
      validityForEvent[event] = callOriginalEventHandlers(event);
    });
    return validityForEvent;
  };
  /**
   * Used to bind an event handler to call the validation logic in addition to its original functionality
   */


  var attachedEventListener = function attachedEventListener(bindEvent) {
    return function (event) {
      var _event$currentTarget;

      for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      if (inputValueParser) {
        setInputValue(inputValueParser.apply(void 0, [event].concat(args)));
      } else if (typeof ((_event$currentTarget = event.currentTarget) === null || _event$currentTarget === void 0 ? void 0 : _event$currentTarget.value) !== 'undefined') {
        setInputValue(event.currentTarget.value);
      } else {
        console.log('Can not find input value to validate against. Using fallback method. Consider using "inputValueParser" to manually define how to parse input value.');
        setInputValue([event].concat(args));
      }

      callOriginalEventHandlers(bindEvent).apply(void 0, [event].concat(args));
    };
  };
  /**
   * Calls the original event handler but attaches UseValidationResponse to the 2nd argument
   */


  var callOriginalEventHandlers = function callOriginalEventHandlers(eventName) {
    return function (event) {
      if (typeof props[eventName] === 'function') {
        var ValidationResponse = {
          validity: validity,
          validate: validate
        };

        for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
          args[_key2 - 1] = arguments[_key2];
        }

        props[eventName].apply(props, [event, ValidationResponse].concat(args));
      }
    };
  };

  var componentProps = _objectSpread(_objectSpread(_objectSpread({}, props), {}, {
    defaultValue: defaultValue
  }, generateAppendValidityEvent()), generateValidationEvent());

  var renderInputComponent = function renderInputComponent() {
    if (typeof InputComponent === 'function') {
      var comp = InputComponent;
      return comp(componentProps);
    } else {
      return /*#__PURE__*/React.createElement(InputComponent, componentProps);
    }
  };

  return /*#__PURE__*/React.createElement(Wrapper, null, renderInputComponent(), children(validity, validate));
};

ValidateInput.defaultProps = defaultProps;
export { ValidateInput };