function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

import { useState, useEffect, useRef } from 'react';

var executeValidators = function executeValidators(inputValue, rules) {
  var messages = [];
  var validationResults = rules.map(function (rule) {
    var verified = rule.validator(inputValue);

    if (!verified) {
      messages.push(rule.message);
    }

    return verified;
  });
  var result = {
    passed: validationResults.every(function (r) {
      return r;
    }),
    messages: messages
  };
  return result;
};

var defaultInitialValidity = {
  passed: true,
  messages: []
};
/**
 * Core logic to parses the validation rules and validates the input value whenever it changes.
 * Returns `{ validity: ValidationState, validate: ValidateFn }`. The `validate` fn is so you can manually trigger the validation logic
 *
 * @param inputValue Value of the input component
 * @param rules Validation rules that applies
 * @param config ValidationConfig:
 * - **inputName?**: string - Name associated with the input component. This is passed as the 3rd argument in onValidity handlers
 * - **validateOnMount?**: boolean - Immediately call the validation fn on mount when true
 * - **onValidityFailure?**: ValidationEventListener - Handler called when validation failed
 * - **onValiditySuccess?**: ValidationEventListener - Handler called when validation passed
 * - **onValidityChanged?**: ValidationEventListener - Handler called when validation state switches
 * - **ValidationEventListener** is defined as `(state: ValidationState, value: any, name?: string) => void`
 *
 * @returns UseValidationResponse:
 * - **validity**: ValidationState - State of the validation result. Defined as `{ passed: boolean, messages: string[] }`
 * - **validate**: ValidateFn - Fn to directly trigger validation logic. Defined as `(inputValue: any, updateState?: boolean) => any`.
 *
 * ***Note***: When `updateState` argument in `validate` fn is `true`, nothing will be returned from the call and will directly update the internal state
 * that tracks the validation state. If `false`, the internal state will NOT be updated, but will return the validation state from the call.
 * This is useful when you want to do the validation check manually before performing another action.
 */

export var useValidation = function useValidation(inputValue, rules, config) {
  var inputName = config.inputName,
      validateOnMount = config.validateOnMount,
      onValidityChanged = config.onValidityChanged,
      onValiditySuccess = config.onValiditySuccess,
      onValidityFailure = config.onValidityFailure;
  var initialValidity = !!validateOnMount ? executeValidators(inputValue, rules) : defaultInitialValidity;

  var _useState = useState([initialValidity, //current
  initialValidity //prev
  ]),
      _useState2 = _slicedToArray(_useState, 2),
      validity = _useState2[0],
      setValidity = _useState2[1];

  var isOnMount = useRef(true);

  var updateValidationState = function updateValidationState(value) {
    return setValidity(function (prev) {
      var state = executeValidators(value, rules);

      if (state.passed !== prev[0].passed && onValidityChanged) {
        onValidityChanged(state, value, inputName);
      }

      if (state.passed && onValiditySuccess) {
        onValiditySuccess(state, value, inputName);
      } else if (!state.passed && onValidityFailure) {
        onValidityFailure(state, value, inputName);
      }

      return [state, prev[0]];
    });
  }; // This is for manual trigger, which may not want to update state and just test if the newest value will pass validation


  var validate = function validate(value) {
    var updateState = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    return updateState ? updateValidationState(value) : executeValidators(value, rules);
  };

  useEffect(function () {
    if (isOnMount.current && !!validateOnMount) {
      updateValidationState(inputValue);
    }
  }, []);
  useEffect(function () {
    if (!isOnMount.current) {
      updateValidationState(inputValue);
    }

    return function () {
      isOnMount.current = false;
    };
  }, [inputValue]);
  return {
    validity: validity[0],
    validate: validate
  };
};